package jesse.keeblarcraft.Commands;

import com.mojang.brigadier.arguments.IntegerArgumentType;
import com.mojang.brigadier.arguments.LongArgumentType;
import com.mojang.brigadier.arguments.StringArgumentType;
import com.mojang.brigadier.context.CommandContext;

import jesse.keeblarcraft.BankMgr.BankManager;
import jesse.keeblarcraft.ConfigMgr.ConfigManager;
import net.minecraft.server.command.CommandManager;
import net.minecraft.server.command.ServerCommandSource;
import net.minecraft.server.network.ServerPlayerEntity;
import net.minecraft.text.Text;
import net.fabricmc.fabric.api.command.v2.CommandRegistrationCallback;

public class BankCommands {
    ConfigManager config = new ConfigManager();

    public void RegisterCommands(){
        // Command: "/getbalance"
        CommandRegistrationCallback.EVENT.register((dispatcher, registryAccess, environment) -> {
            dispatcher.register(CommandManager.literal("getbalance")
            .executes(context -> GetBalance(context)));
        });

        // Command: "/setbalance"
        CommandRegistrationCallback.EVENT.register((dispatcher, registryAccess, environment) -> {
            final var rootSetBalance = dispatcher.register(CommandManager.literal("setbalance")
            .then(CommandManager.argument("newBalance", IntegerArgumentType.integer())
            .then(CommandManager.argument("reason", StringArgumentType.string())
            .then(CommandManager.argument("payment", LongArgumentType.longArg())
            .then(CommandManager.argument("otherParty", StringArgumentType.string())
            //probably will remove time and otherParty from this agurment list not sure yet
            .then(CommandManager.argument("time", IntegerArgumentType.integer())
            .executes(context -> SetBalance(
                IntegerArgumentType.getInteger(context, "newBalance"),
                StringArgumentType.getString(context, "reason"),
                LongArgumentType.getLong(context, "payment"),
                StringArgumentType.getString(context, "otherParty"),
                IntegerArgumentType.getInteger(context, "time"),
            context))))))));

            dispatcher.register(CommandManager.literal("resetbalance").redirect(rootSetBalance));
        });

        // Command: "/AddMoney"
        CommandRegistrationCallback.EVENT.register((dispatcher, registryAccess, environment) -> {
            dispatcher.register(CommandManager.literal("AddMoney")
            .then(CommandManager.argument("newBalance", IntegerArgumentType.integer())
            .then(CommandManager.argument("reason", StringArgumentType.string())
            .then(CommandManager.argument("payment", LongArgumentType.longArg())
            .then(CommandManager.argument("otherParty", StringArgumentType.string())
            //probably will remove time and otherParty from this agurment list not sure yet
            .then(CommandManager.argument("time", IntegerArgumentType.integer())
            .executes(context -> AddMoney(
                IntegerArgumentType.getInteger(context, "newBalance"),
                StringArgumentType.getString(context, "reason"),
                LongArgumentType.getLong(context, "payment"),
                StringArgumentType.getString(context, "otherParty"),
                IntegerArgumentType.getInteger(context, "time"),
            context))))))));
        });

        // Command: "/removebalance"
        CommandRegistrationCallback.EVENT.register((dispatcher, registryAccess, environment) -> {
            final var rootSubtractBalance = dispatcher.register(CommandManager.literal("subtractbalance")
            .then(CommandManager.argument("newBalance", IntegerArgumentType.integer())
            .then(CommandManager.argument("reason", StringArgumentType.string())
            .then(CommandManager.argument("payment", LongArgumentType.longArg())
            .then(CommandManager.argument("otherParty", StringArgumentType.string())
            //probably will remove time and otherParty from this agurment list not sure yet
            .then(CommandManager.argument("time", IntegerArgumentType.integer())
            .executes(context -> SubtractBalance(
                IntegerArgumentType.getInteger(context, "newBalance"),
                StringArgumentType.getString(context, "reason"),
                LongArgumentType.getLong(context, "payment"),
                StringArgumentType.getString(context, "otherParty"),
                IntegerArgumentType.getInteger(context, "time"),
            context))))))));

            dispatcher.register(CommandManager.literal("-balance").redirect(rootSubtractBalance));
        });

        // Command: "/wire"
        CommandRegistrationCallback.EVENT.register((dispatcher, registryAccess, environment) -> {
            final var sendmoney = dispatcher.register(CommandManager.literal("wire")
            .then(CommandManager.argument("newBalance", IntegerArgumentType.integer())
            .then(CommandManager.argument("reason", StringArgumentType.string())
            .then(CommandManager.argument("payment", LongArgumentType.longArg())
            .then(CommandManager.argument("otherParty", StringArgumentType.string())
            //probably will remove time and otherParty from this agurment list not sure yet
            .then(CommandManager.argument("time", IntegerArgumentType.integer())
            .executes(context -> Wire(
                IntegerArgumentType.getInteger(context, "newBalance"),
                StringArgumentType.getString(context, "reason"),
                LongArgumentType.getLong(context, "payment"),
                StringArgumentType.getString(context, "otherParty"),
                IntegerArgumentType.getInteger(context, "time"),
            context))))))));

            dispatcher.register(CommandManager.literal("resetbalance").redirect(sendmoney));
        });
    }

    public Integer GetBalance(CommandContext<ServerCommandSource> context) {
        Integer ret = -1;
        
        if (context.getSource().isExecutedByPlayer()) {
            ServerPlayerEntity player = context.getSource().getPlayer();
            BankManager playerBank = new BankManager(player.getUuidAsString());
            player.sendMessage(Text.literal(String.valueOf(playerBank.GetBalance())));
            return 0;
        }
        
        return ret;
    }

    //SetBalance will be a ServerCommand only
    public Integer SetBalance(Integer newBalance, String reason, long payment, String otherParty, Integer time, CommandContext<ServerCommandSource> context) {
        Integer ret = -1;

        if (context.getSource().isExecutedByPlayer()) {
            ServerPlayerEntity player = context.getSource().getPlayer();
            BankManager playerBank = new BankManager(player.getUuidAsString());
            playerBank.SetBalance(newBalance, reason, payment, otherParty, time);
            player.sendMessage(Text.literal(String.valueOf(playerBank.GetBalance())));
        }

        return ret;
    }

    //AddMoney will be a ServerCommand Only
    public Integer AddMoney(Integer newBalance, String reason, long payment, String otherParty, Integer time, CommandContext<ServerCommandSource> context) {
        Integer ret = -1;

        if (context.getSource().isExecutedByPlayer()) {
            ret = 0;
            ServerPlayerEntity player = context.getSource().getPlayer();
            BankManager playerBank = new BankManager(player.getUuidAsString());
            playerBank.AddMoney(newBalance, reason, payment, otherParty, time);
            player.sendMessage(Text.literal(String.valueOf(playerBank.GetBalance())));
        }

        return ret;
    }

    //SubtractBalance will be a ServerCommand only
    public Integer SubtractBalance(Integer newBalance, String reason, long payment, String otherParty, Integer time, CommandContext<ServerCommandSource> context) {
        Integer ret = -1;

        if (context.getSource().isExecutedByPlayer()) {
            ret = 0;
            ServerPlayerEntity player = context.getSource().getPlayer();
            BankManager playerBank = new BankManager(player.getUuidAsString());
            playerBank.SubtractBalance(newBalance, reason, payment, otherParty, time);
            player.sendMessage(Text.literal(String.valueOf(playerBank.GetBalance())));
        }

        return ret;
    }
    
    public Integer Wire(Integer newBalance, String reason, long payment, String otherParty, Integer time, CommandContext<ServerCommandSource> context) {
        Integer ret = -1;

        if (context.getSource().isExecutedByPlayer()) {
            ret = 0;
            ServerPlayerEntity player = context.getSource().getPlayer();
            BankManager playerBank = new BankManager(player.getUuidAsString());
            playerBank.Wire(newBalance, reason, payment, otherParty, time);
            player.sendMessage(Text.literal(String.valueOf(playerBank.GetBalance())));
        }

        return ret;
    }
}
